@@ -8,8 +8,8 @@ from buildbot.config import BuilderConfig
 
 from buildbot import locks
 from buildbot.process.factory import BuildFactory
+from steps import PgSetProperties
 from buildbot.steps.shell import ShellCommand
-from buildbot.steps.shell import SetProperty
 from buildbot.steps.transfer import FileDownload
 from buildbot.process.properties import WithProperties
 from buildbot.process.properties import Property
@@ -27,6 +27,8 @@ BUILDSLAVE_KWARGS = { # name -> validating callable
 
 BUILDSLAVE_REQUIRED = ('password',)
 
+BUILD_UTILS_PATH = os.path.join(os.path.split(__file__)[0], 'build_utils')
+
 logger = logging.getLogger(__name__)
 
 # Running buildouts in parallel on one slave fails
@@ -34,18 +36,28 @@ logger = logging.getLogger(__name__)
 buildout_lock = locks.SlaveLock("buildout")
 
 class BuildoutsConfigurator(object):
-    """Populate buildmaster configs from buildouts and external slaves.cfg"""
+    """Populate buildmaster configs from buildouts and external slaves.cfg.
 
-    def __init__(self, master_cfg_file):
+    There are two dictionnaries of methods:
+       post_buildout_steps : name -> method returning a list of steps to be
+                             added after buildout has run and testing db is
+                             done
+       buildout_dl_steps : name -> method returning a list of steps to
+                           construct the buildout configuration slave-side.
+    """
+
+    def __init__(self, buildmaster_dir,
+                 manifest_paths=('buildouts/MANIFEST.cfg',)):
         """Attach to buildmaster in which master_cfg_file path sits.
         """
-        self.buildmaster_dir = os.path.split(master_cfg_file)[0]
+        self.buildmaster_dir = buildmaster_dir
         self.build_factories = {} # build factories by name
         self.factories_to_builders = {} # factory name -> builders playing it
+        self.manifest_paths = manifest_paths
 
     def populate(self, config):
         config.setdefault('slaves', []).extend(self.make_slaves('slaves.cfg'))
-        self.register_build_factories('buildouts/MANIFEST.cfg')
+        map(self.register_build_factories, self.manifest_paths)
         config.setdefault('builders', []).extend(
             self.make_builders(master_config=config))
         config.setdefault('schedulers', []).extend(self.make_schedulers())
@@ -116,40 +128,47 @@ class BuildoutsConfigurator(object):
 
         return slaves
 
-    def buildout_standalone_dl_steps(self, cfg_tokens):
+    def buildout_standalone_dl_steps(self, cfg_tokens, manifest_dir):
         """Return slave side path and steps about the buildout.
 
         The first returned value is the expected path from build directory
         The second is an iterable of steps to get the buildout config file
         and the related needed files (extended cfgs, bootstrap.py).
+
+        manifest_dir is the path (interpreted from buildmaster dir) to the
+        directory in with the manifest file sits.
         """
         if len(cfg_tokens) != 1:
             raise ValueError(
-                "Wrong standalong buildout specification: %r" % tokens)
+                "Wrong standalong buildout specification: %r" % cfg_tokens)
 
         conf_path = cfg_tokens[0]
         conf_name = os.path.split(conf_path)[-1]
-        return conf_name, (FileDownload(mastersrc='buildouts/bootstrap.py',
+        conf_path = os.path.join(manifest_dir, conf_path)
+        bootstrap_path = os.path.join(manifest_dir, 'bootstrap.py')
+        return conf_name, (FileDownload(mastersrc=bootstrap_path,
                                         slavedest='bootstrap.py'),
                            FileDownload(mastersrc=conf_path,
                                         slavedest=conf_name),
                            )
 
-    def buildout_hg_dl_steps(self, cfg_tokens):
+    def buildout_hg_dl_steps(self, cfg_tokens, manifest_dir):
         """Return slave side path and steps about the buildout.
 
         The first returned value is the expected path from build directory
         The second is an iterable of steps to get the buildout config file
         and the related needed files (extended cfgs, bootstrap.py).
+
+        manifest_dir is not used in this downloader.
         """
         if len(cfg_tokens) != 3:
             raise ValueError(
-                "Wrong standalong buildout specification: %r" % tokens)
+                "Wrong standalong buildout specification: %r" % cfg_tokens)
 
         url, branch, conf_path = cfg_tokens
         return conf_path, (
             FileDownload(
-                mastersrc='build_utils/buildout_hg_dl.py',
+                mastersrc=os.path.join(BUILD_UTILS_PATH, 'buildout_hg_dl.py'),
                 slavedest='buildout_hg_dl.py',
                 haltOnFailure=True),
             ShellCommand(
@@ -184,43 +203,17 @@ class BuildoutsConfigurator(object):
         for dl_step in buildout_dl_steps:
             factory.addStep(dl_step)
 
-        factory.addStep(FileDownload(mastersrc='build_utils/'
-                                     'analyze_oerp_tests.py',
-                                     slavedest='analyze_oerp_tests.py'))
-
-        def set_pg_cluster_props(step):
-            """Set postgresql cluster properties
-
-            All properties defined in the slave capability line for the
-            builder-level ``pg_version`` property are applied to the build,
-            with pg_prefix.
-
-            Example: port=5434 gives pg_port=5434
-
-            GR XXX Of course it is quite hacky to rely on side effect of a
-            doStepIf option. This is the quickest working solution I could come
-            with. Better to hook this in the build definition stage itself.
-            """
-            pg_version = step.getProperty('pg_version')
-            pg_props = step.getProperty('capability')['postgresql'][pg_version]
-            for k, v in pg_props.items():
-                step.setProperty('pg_%s' % k, v)
-            return True
-
-        factory.addStep(ShellCommand(
-            command=['/bin/echo',
-                     WithProperties('capability: %(capability)s')],
-            doStepIf=set_pg_cluster_props,
-            description=["Setting", "pg cluster", "properties"],
-            descriptionDone=["Set", "pg cluster", "properties"],
+        factory.addStep(FileDownload(
+                mastersrc=os.path.join(
+                    BUILD_UTILS_PATH, 'analyze_oerp_tests.py'),
+                slavedest='analyze_oerp_tests.py'))
+        factory.addStep(PgSetProperties(
+            description=["Setting", "PG cluster", "properties"],
+            descriptionDone=["Set", "PG cluster", "properties"],
             name="pg_cluster_props",
+            factory_name=name,
             ))
 
-        factory.addStep(ShellCommand(command=['python', 'bootstrap.py',
-                                              '-c', buildout_slave_path],
-                                     haltOnFailure=True,
-                                     ))
-
         cache = '../../buildout-caches'
         eggs_cache = cache + '/eggs'
         openerp_cache = cache + '/openerp'
@@ -229,6 +222,11 @@ class BuildoutsConfigurator(object):
                                      name="cachedirs",
                                      description="prepare cache dirs"))
 
+        factory.addStep(ShellCommand(command=['python', 'bootstrap.py',
+                                              '-c', buildout_slave_path],
+                                     haltOnFailure=True,
+                                     ))
+
         psycopg2_env=dict(PATH=[WithProperties('%(pg_bin:-)s'),
                                 '${PATH}'],
                           LD_LIBRARY_PATH=WithProperties('%(pg_lib:-)s'),
@@ -266,11 +264,6 @@ class BuildoutsConfigurator(object):
                               '${PATH}'],
                         )
 
-        factory.addStep(SetProperty(
-                property='testing_db',
-                command=WithProperties(
-                    "echo %(db_prefix:-openerp-buildbot)s-" + name)))
-
         factory.addStep(ShellCommand(command=[
                     psql, 'postgres', '-c',
                     WithProperties('DROP DATABASE IF EXISTS "%(testing_db)s"'),
@@ -282,7 +275,9 @@ class BuildoutsConfigurator(object):
                                      haltOnFailure=True))
         factory.addStep(ShellCommand(command=[
                     psql, 'postgres', '-c',
-                    WithProperties('CREATE DATABASE "%(testing_db)s"'),
+                    WithProperties(
+            'CREATE DATABASE "%%(testing_db)s" '
+            'TEMPLATE "%s"' % options.get('db_template', 'template1')),
                     ],
                                      name='createdb',
                                      description=["createdb",
@@ -290,32 +285,15 @@ class BuildoutsConfigurator(object):
                                      env=psql_env,
                                      haltOnFailure=True,
                                      ))
-        factory.addStep(ShellCommand(command=['rm', '-f', 'test.log'],
-                                     name="Log cleanup",
-                                     descriptionDone=['Cleaned', 'logs'],
-                                     ))
 
-        factory.addStep(ShellCommand(command=[
-                    'bin/start_openerp', '-i',
-                    comma_list_sanitize(options.get('openerp-addons', 'all')),
-                    '--stop-after-init',
-                    '--log-level=test', '-d', Property('testing_db'),
-                    # openerp --logfile does not work with relative paths !
-                    WithProperties('--logfile=%(workdir)s/build/test.log')],
-                                     name='testing',
-                                     description='testing',
-                                     descriptionDone='tests',
-                                     logfiles=dict(test='test.log'),
-                                     haltOnFailure=True,
-                                     env=psycopg2_env,
-                                     )),
-
-        factory.addStep(ShellCommand(
-                command=["python", "analyze_oerp_tests.py", "test.log"],
-                name='analyze',
-                description="analyze",
-                ))
+        post_buildout_steps = self.post_buildout_steps[
+            options.get('post-buildout-steps', 'standard')]
+        for step in post_buildout_steps(self, options, buildout_slave_path,
+                                        psycopg2_env=psycopg2_env):
+            factory.addStep(step)
 
+        # TODO GR this is outside of the factory itself and get back
+        # to the caller
         build_for = options.get('build-for')
         factory.build_for = {}
         if build_for is not None:
@@ -335,6 +313,44 @@ class BuildoutsConfigurator(object):
             factory.build_category = build_category.strip()
         return factory
 
+    def post_buildout_steps_standard(self, options, buildout_slave_path,
+                                     psycopg2_env=None):
+
+        if psycopg2_env is None:
+            psycopg2_env = {}
+
+        steps = []
+
+        steps.append(ShellCommand(command=['rm', '-f', 'test.log'],
+                                     name="Log cleanup",
+                                     descriptionDone=['Cleaned', 'logs'],
+                                     ))
+
+        steps.append(ShellCommand(command=[
+                    'bin/start_openerp', '-i',
+                    comma_list_sanitize(options.get('openerp-addons', 'all')),
+                    '--stop-after-init',
+                    '--log-level=test', '-d', Property('testing_db'),
+                    # openerp --logfile does not work with relative paths !
+                    WithProperties('--logfile=%(workdir)s/build/test.log')],
+                                     name='testing',
+                                     description='testing',
+                                     descriptionDone='tests',
+                                     logfiles=dict(test='test.log'),
+                                     haltOnFailure=True,
+                                     env=psycopg2_env,
+                                     )),
+
+        steps.append(ShellCommand(
+                command=["python", "analyze_oerp_tests.py", "test.log"],
+                name='analyze',
+                description="analyze",
+                ))
+
+        return steps
+
+    post_buildout_steps = dict(standard=post_buildout_steps_standard)
+
     def register_build_factories(self, manifest_path):
         """Register a build factory per buildout from file at manifest_path.
 
@@ -348,6 +364,7 @@ class BuildoutsConfigurator(object):
         """
         parser = ConfigParser()
         parser.read(self.path_from_buildmaster(manifest_path))
+        manifest_dir = os.path.split(manifest_path)[0]
         registry = self.build_factories
 
         for name in parser.sections():
@@ -363,9 +380,11 @@ class BuildoutsConfigurator(object):
                 raise ValueError("Buildout type %r in %r not supported" % (
                         btype, name))
 
-            conf_slave_path, dl_steps = buildout_downloader(self, buildout[1:])
-            registry[name] = self.make_factory(name, conf_slave_path, dl_steps,
-                                               dict(parser.items(name)))
+            conf_slave_path, dl_steps = buildout_downloader(self, buildout[1:],
+                                                            manifest_dir)
+            registry[name] = factory = self.make_factory(
+                name, conf_slave_path, dl_steps, dict(parser.items(name)))
+            factory.manifest_path = manifest_path # change filter will need it
 
     buildout_dl_steps = dict(standalone=buildout_standalone_dl_steps,
                              hg=buildout_hg_dl_steps)
@@ -421,6 +440,14 @@ class BuildoutsConfigurator(object):
             all_builders.extend(builders)
         return all_builders
 
+    def factory_to_manifest(self, fact_name, absolute=False):
+        """Return the path to manifest file where factory with given name arose.
+        """
+        path = self.build_factories[fact_name].manifest_path
+        if absolute:
+            path = self.path_from_buildmaster(path)
+        return path
+
     def make_schedulers(self):
         """We make one scheduler per build factory (ie per buildout).
 
@@ -429,9 +456,14 @@ class BuildoutsConfigurator(object):
         not be preferable for buildmaster performance.
         """
         fact_to_builders = self.factories_to_builders
+        def make_filter(factory_name):
+            """Make a Mirror Change Filter for factory with given name."""
+            return MirrorChangeFilter(
+                self.factory_to_manifest(factory_name, absolute=True),
+                factory_name)
+
         return [SingleBranchScheduler(name=factory_name,
-                                      change_filter=MirrorChangeFilter(
-                    self.buildmaster_dir, factory_name),
-                                      treeStableTimer=600,
+                                      change_filter=make_filter(factory_name),
+                                      treeStableTimer=60,
                                       builderNames=builders)
                 for factory_name, builders in fact_to_builders.items()]
